(function ($, Drupal) {
  'use strict';

  /**
   * Cookie content blocker behavior for loading blocked content and scripts.
   */
  Drupal.behaviors.cookieContentBlocker = {
    initialized: false,
    consent: (Drupal.behaviors.cookieContentBlocker !== void (0) && Drupal.behaviors.cookieContentBlocker.consent !== void (0)) ?
      Drupal.behaviors.cookieContentBlocker.consent :
      false,

    /**
     * Initialize after the page is loaded and update after AJAX requests.
     *
     * @param {HTMLElement} context
     *   The attached context.
     * @param {object} settings
     *   The Drupal JS settings.
     */
    attach: function (context, settings) {
      var $window = $(window);
      var $blockers = $('.js-cookie-content-blocker', context);
      var cookieContentBlocker = Drupal.behaviors.cookieContentBlocker;

      // Handle already accepted cookies.
      // @todo add logic for a generic cookie API.
      if (cookieContentBlocker.consent) {
        cookieContentBlocker.handleCookieAccepted(context, settings);
      }

      $blockers
        .find('.js-cookie-content-blocker-consent-change-button, .js-cookie-content-blocker-click-consent-change')
        .once(function () {
          $(this).on('click', function (event) {
            event.preventDefault();
            // @todo add generic cookie API events when available.
            $window.trigger('cookieContentBlockerChangeConsent');
          });
        });

      if (cookieContentBlocker.initialized) {
        return;
      }

      cookieContentBlocker.initialized = true;

      // @todo add generic cookie API events when available.
      $window.on('cookieContentBlockerConsentGiven', function () {
        cookieContentBlocker.consent = true;
        // @todo note context and settings stay the same after AJAX, test.
        cookieContentBlocker.handleCookieAccepted(context, settings);
      });
    },

    /**
     * Handle the fact that cookies are accepted by the user.
     *
     * @param {HTMLElement} context
     *   The attached context.
     * @param {object} settings
     *   The Drupal JS settings.
     */
    handleCookieAccepted: function (context, settings) {
      Drupal.behaviors.cookieContentBlocker.loadBlockedAttachedScripts(context, settings);
      Drupal.behaviors.cookieContentBlocker.loadBlockedContent(context, settings);
    },

    /**
     * Loads the blocked content in place.
     *
     * @param {HTMLElement} context
     *   The attached context.
     * @param {object} settings
     *   The Drupal JS settings.
     */
    loadBlockedContent: function (context, settings) {
      $('.js-cookie-content-blocker-content', context).each(function () {
        var $originalContentWrapperScript = $(this);
        var $blocker = $originalContentWrapperScript.closest('.js-cookie-content-blocker');
        var originalContent = $originalContentWrapperScript.text();
        // Replace the <scriptfake> placeholder tags with real script tags.
        // See: _cookie_content_blocker_replace_scripts_with_fake() and
        // cookie-content-blocker-wrapper.tpl.php.
        originalContent = originalContent.replace(new RegExp(/(<[/]?script)fake/, 'g'), '$1');

        if ($blocker.length) {
          $blocker.replaceWith(originalContent);
          return;
        }

        $originalContentWrapperScript.replaceWith(originalContent);
      });
    },

    /**
     * Loads the blocked attached scripts.
     *
     * @param {HTMLElement} context
     *   The attached context.
     * @param {object} settings
     *   The Drupal JS settings.
     */
    loadBlockedAttachedScripts: function (context, settings) {
      if (settings.cookieContentBlocker === void (0) || settings.cookieContentBlocker.scripts === void (0)) {
        return;
      }

      $('[data-cookie-content-blocker-script-id]').each(function () {
        var $blockedScript = $(this);
        var id = $blockedScript.data('cookie-content-blocker-script-id');

        if (!(id in settings.cookieContentBlocker.scripts)) {
          return;
        }

        $blockedScript.replaceWith(settings.cookieContentBlocker.scripts[id]);
      });
    }

  };

})(jQuery, Drupal);
